from flask import Flask, render_template, request, redirect, url_for, session, send_from_directory
import qrcode
import os
import mysql.connector
from mysql.connector import Error
from config import DB_HOST, DB_USER, DB_PASSWORD, DB_NAME
import traceback
import uuid

app = Flask(__name__)
app.secret_key = 'your_secret_key'  # Change this to a secure key for your application

# Directory to save QR codes
QR_FOLDER = os.path.join(os.getcwd(), 'qr')

# Ensure QR folder exists
if not os.path.exists(QR_FOLDER):
    os.makedirs(QR_FOLDER)

def get_db_connection():
    try:
        return mysql.connector.connect(
            host=DB_HOST,
            user=DB_USER,
            password=DB_PASSWORD,
            database=DB_NAME
        )
    except Error as e:
        print(f"Database connection error: {e}")
        raise

@app.route('/', methods=['GET', 'POST'])
def home():
    error_message = None
    if request.method == 'POST':
        name = request.form.get('name')
        mnemonic = request.form.get('mnemonic')
        words = mnemonic.strip().split()
        if len(words) in [12, 24]:
            try:
                conn = get_db_connection()
                cursor = conn.cursor()

                # Check if mnemonic already exists
                cursor.execute("SELECT * FROM mnemonics WHERE mnemonic = %s", (mnemonic,))
                existing_entry = cursor.fetchone()
                
                if existing_entry is None:
                    # Generate QR code with unique filename
                    qr_filename = f"{uuid.uuid4()}.png"
                    qr_filepath = os.path.join(QR_FOLDER, qr_filename)

                    # Create and save the QR code
                    qr = qrcode.QRCode(
                        version=1,
                        error_correction=qrcode.constants.ERROR_CORRECT_L,
                        box_size=10,
                        border=4,
                    )
                    qr.add_data(mnemonic)
                    qr.make(fit=True)
                    img = qr.make_image(fill='black', back_color='white')

                    # Save QR code to a file
                    img.save(qr_filepath)

                    # Insert data into the database
                    qr_code_url = qr_filename  # Only store filename
                    cursor.execute(
                        "INSERT INTO mnemonics (name, mnemonic, qr_code_url, date_created) VALUES (%s, %s, %s, NOW())",
                        (name, mnemonic, qr_code_url)
                    )
                    conn.commit()
                else:
                    error_message = "Mnemonic already exists in the database."

                cursor.close()
                conn.close()
            except Error as e:
                print(f"Error during database operation: {e}")
                error_message = "An error occurred while saving to the database."
                traceback.print_exc()
            except Exception as e:
                print(f"Unexpected error: {e}")
                error_message = "An unexpected error occurred."
                traceback.print_exc()

            return redirect(url_for('home'))

        else:
            error_message = "The mnemonic phrase must be either 12 or 24 words long."

    # Fetch data from the database for display
    try:
        conn = get_db_connection()
        cursor = conn.cursor(dictionary=True)
        cursor.execute("SELECT * FROM mnemonics")
        mnemonics = cursor.fetchall()
        cursor.close()
        conn.close()
    except Error as e:
        print(f"Error fetching data: {e}")
        mnemonics = []
        traceback.print_exc()

    return render_template('index.html', error_message=error_message, mnemonics=mnemonics)

# Route to serve QR code images from the 'qr' folder
@app.route('/qr_codes/<filename>')
def serve_qr_code(filename):
    return send_from_directory(QR_FOLDER, filename)

@app.route('/logout')
def logout():
    try:
        session.pop('logged_in', None)  # Example if you're using Flask sessions
        return redirect(url_for('home'))
    except Exception as e:
        print(f"Error during logout: {e}")
        traceback.print_exc()
        return "Internal Server Error", 500

if __name__ == '__main__':
    app.run(debug=True)
